@@ -17,24 +17,21 @@
 // Kaidan
 #include "AccountDb.h"
 #include "AtmController.h"
-#include "AvatarCache.h"
 #include "Blocking.h"
 #include "ChatController.h"
 #include "CredentialsValidator.h"
+#include "DiscoveryController.h"
 #include "EncryptionController.h"
 #include "FileSharingController.h"
 #include "FutureUtils.h"
 #include "GroupChatController.h"
 #include "LogHandler.h"
-#include "MainController.h"
 #include "MessageController.h"
-#include "MessageDb.h"
 #include "NotificationController.h"
 #include "PresenceCache.h"
 #include "QmlUtils.h"
 #include "RegistrationController.h"
 #include "RosterController.h"
-#include "RosterDb.h"
 #include "RosterModel.h"
 #include "SystemUtils.h"
 #include "VCardController.h"
@@ -205,6 +202,19 @@ quint16 AccountSettings::autoDetectPort() const
     return AUTO_DETECT_PORT;
 }
 
+bool AccountSettings::plainAuthAllowed() const
+{
+    return m_data.plainAuthAllowed;
+}
+
+void AccountSettings::setPlainAuthAllowed(bool plainAuthAllowed)
+{
+    if (m_data.plainAuthAllowed != plainAuthAllowed) {
+        m_data.plainAuthAllowed = plainAuthAllowed;
+        Q_EMIT plainAuthAllowedChanged();
+    }
+}
+
 QString AccountSettings::name() const
 {
     return m_data.name;
@@ -273,6 +283,32 @@ QString AccountSettings::httpUploadLimitText() const
     return QmlUtils::formattedDataSize(m_data.httpUploadLimit);
 }
 
+void AccountSettings::setChatSupportAddresses(const QList<QString> &chatSupportAddresses)
+{
+    if (m_data.chatSupportAddresses != chatSupportAddresses) {
+        m_data.chatSupportAddresses = chatSupportAddresses;
+        Q_EMIT chatSupportAddressesChanged();
+    }
+}
+
+QList<QString> AccountSettings::chatSupportAddresses() const
+{
+    return m_data.chatSupportAddresses;
+}
+
+void AccountSettings::setGroupChatSupportAddresses(const QList<QString> &groupChatSupportAddresses)
+{
+    if (m_data.groupChatSupportAddresses != groupChatSupportAddresses) {
+        m_data.groupChatSupportAddresses = groupChatSupportAddresses;
+        Q_EMIT groupChatSupportAddressesChanged();
+    }
+}
+
+QList<QString> AccountSettings::groupChatSupportAddresses() const
+{
+    return m_data.groupChatSupportAddresses;
+}
+
 AccountSettings::PasswordVisibility AccountSettings::passwordVisibility() const
 {
     return m_data.passwordVisibility;
@@ -385,6 +421,7 @@ void AccountSettings::storeTemporaryData()
             account.userAgentDeviceId = data.userAgentDeviceId;
             account.host = data.host;
             account.port = data.port;
+            account.plainAuthAllowed = data.plainAuthAllowed;
             account.name = data.name;
         });
     } else {
@@ -394,6 +431,22 @@ void AccountSettings::storeTemporaryData()
     }
 }
 
+QString AccountSettings::loginUriString() const
+{
+    QXmpp::Uri::Login loginQuery;
+
+    if (passwordVisibility() != AccountSettings::PasswordVisibility::Invisible) {
+        loginQuery.password = password();
+    }
+
+    QXmppUri uri;
+
+    uri.setJid(jid());
+    uri.setQuery(std::move(loginQuery));
+
+    return uri.toString();
+}
+
 void AccountSettings::generateJidResource()
 {
     m_data.jidResource = QXmppUtils::generateStanzaHash(4);
@@ -464,7 +517,7 @@ QString Connection::errorText() const
     case ClientWorker::ConnectionRefused:
         return tr("The server is offline or blocked by a firewall.");
     case ClientWorker::NoSupportedAuth:
-        return tr("Authentification protocol not supported by the server.");
+        return tr("Authentication protocol not supported by the server.");
     case ClientWorker::KeepAliveError:
         return tr("The connection could not be refreshed.");
     case ClientWorker::NoNetworkPermission:
@@ -527,75 +580,32 @@ Account::Account(AccountSettings::Data accountSettingsData, QObject *parent)
         return XmppClient{thread, worker};
     }())
     , m_connection(new Connection(m_client.worker, this))
-    , m_avatarCache(new AvatarCache(this))
     , m_presenceCache(new PresenceCache(this))
     , m_atmController(new AtmController(m_client.worker->atmManager(), this))
     , m_blockingController(new BlockingController(m_settings, m_connection, m_client.worker, this))
     , m_encryptionController(new EncryptionController(m_settings, m_presenceCache, m_client.worker->omemoManager(), this))
-    , m_fileSharingController(new FileSharingController(m_settings, m_connection, m_client.worker, this))
     , m_rosterController(new RosterController(m_settings, m_connection, m_encryptionController, m_client.worker->rosterManager(), this))
+    , m_fileSharingController(new FileSharingController(m_settings, m_connection, m_client.worker, this))
     , m_messageController(new MessageController(m_settings,
                                                 m_connection,
                                                 m_encryptionController,
                                                 m_rosterController,
+                                                m_fileSharingController,
                                                 m_client.worker->xmppClient(),
                                                 m_client.worker->mamManager(),
                                                 m_client.worker->messageReceiptManager(),
                                                 this))
     , m_groupChatController(new GroupChatController(m_settings, m_messageController, m_client.worker->mixManager(), this))
     , m_notificationController(new NotificationController(m_settings, m_messageController, this))
-    , m_vCardController(
-          new VCardController(m_settings, m_connection, m_avatarCache, m_presenceCache, m_client.worker->xmppClient(), m_client.worker->vCardManager(), this))
+    , m_vCardController(new VCardController(m_settings, m_connection, m_presenceCache, m_client.worker->xmppClient(), m_client.worker->vCardManager(), this))
     , m_registrationController(new RegistrationController(m_settings, m_connection, m_encryptionController, m_vCardController, m_client.worker, this))
     , m_versionController(new VersionController(m_presenceCache, m_client.worker->versionManager()))
 {
+    new DiscoveryController(m_settings, m_connection, m_client.worker->discoveryManager(), this);
+
     runOnThread(m_client.worker, [this]() {
         m_client.worker->initialize(m_atmController, m_encryptionController, m_messageController, m_registrationController, m_presenceCache);
     });
-
-    connect(m_avatarCache, &AvatarCache::avatarIdsChanged, this, &Account::avatarCacheChanged);
-
-    connect(MessageDb::instance(), &MessageDb::messageAdded, this, [this](const Message &message, MessageOrigin origin) {
-        if (origin != MessageOrigin::UserInput && message.accountJid == m_settings->jid() && !message.files.isEmpty()) {
-            if (const auto item = RosterModel::instance()->item(message.accountJid, message.chatJid)) {
-                const auto contactRule = item->automaticMediaDownloadsRule;
-
-                const auto effectiveRule = [this, contactRule]() -> AccountSettings::AutomaticMediaDownloadsRule {
-                    switch (contactRule) {
-                    case RosterItem::AutomaticMediaDownloadsRule::Account:
-                        return m_settings->automaticMediaDownloadsRule();
-                    case RosterItem::AutomaticMediaDownloadsRule::Never:
-                        return AccountSettings::AutomaticMediaDownloadsRule::Never;
-                    case RosterItem::AutomaticMediaDownloadsRule::Always:
-                        return AccountSettings::AutomaticMediaDownloadsRule::Always;
-                    }
-
-                    Q_UNREACHABLE();
-                }();
-
-                const auto automaticDownloadDesired = [effectiveRule, &message]() -> bool {
-                    switch (effectiveRule) {
-                    case AccountSettings::AutomaticMediaDownloadsRule::Never:
-                        return false;
-                    case AccountSettings::AutomaticMediaDownloadsRule::PresenceOnly:
-                        return message.isOwn || RosterModel::instance()->isPresenceSubscribedByItem(message.accountJid, message.chatJid);
-                    case AccountSettings::AutomaticMediaDownloadsRule::Always:
-                        return true;
-                    }
-
-                    Q_UNREACHABLE();
-                }();
-
-                if (automaticDownloadDesired) {
-                    for (const auto &file : message.files) {
-                        if (file.localFilePath.isEmpty() || !QFile::exists(file.localFilePath)) {
-                            m_fileSharingController->downloadFile(message.chatJid, message.id, file);
-                        }
-                    }
-                }
-            }
-        }
-    });
 }
 
 Account::~Account()
@@ -676,11 +686,6 @@ VersionController *Account::versionController() const
     return m_versionController;
 }
 
-AvatarCache *Account::avatarCache() const
-{
-    return m_avatarCache;
-}
-
 PresenceCache *Account::presenceCache() const
 {
     return m_presenceCache;
