@@ -21,7 +21,6 @@
 constexpr quint16 AUTO_DETECT_PORT = 0;
 
 class AtmController;
-class AvatarCache;
 class BlockingController;
 class ChatController;
 class ClientWorker;
@@ -47,12 +46,15 @@ class AccountSettings : public QObject
     Q_PROPERTY(QString host READ host WRITE setHost NOTIFY hostChanged)
     Q_PROPERTY(quint16 port READ port WRITE setPort NOTIFY portChanged)
     Q_PROPERTY(quint16 autoDetectPort READ autoDetectPort CONSTANT FINAL)
+    Q_PROPERTY(bool plainAuthAllowed READ plainAuthAllowed WRITE setPlainAuthAllowed NOTIFY plainAuthAllowedChanged)
     Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
     Q_PROPERTY(QString displayName READ displayName NOTIFY nameChanged)
     Q_PROPERTY(bool enabled READ enabled NOTIFY enabledChanged)
     Q_PROPERTY(bool inBandRegistrationFeaturesSupported READ inBandRegistrationFeaturesSupported NOTIFY inBandRegistrationFeaturesSupportedChanged)
     Q_PROPERTY(qint64 httpUploadLimit READ httpUploadLimit NOTIFY httpUploadLimitChanged)
     Q_PROPERTY(QString httpUploadLimitText READ httpUploadLimitText NOTIFY httpUploadLimitChanged)
+    Q_PROPERTY(QList<QString> chatSupportAddresses READ chatSupportAddresses NOTIFY chatSupportAddressesChanged)
+    Q_PROPERTY(QList<QString> groupChatSupportAddresses READ groupChatSupportAddresses NOTIFY groupChatSupportAddressesChanged)
     Q_PROPERTY(AccountSettings::PasswordVisibility passwordVisibility READ passwordVisibility WRITE setPasswordVisibility NOTIFY passwordVisibilityChanged)
     Q_PROPERTY(Encryption::Enum encryption READ encryption WRITE setEncryption NOTIFY encryptionChanged)
     Q_PROPERTY(AccountSettings::AutomaticMediaDownloadsRule automaticMediaDownloadsRule READ automaticMediaDownloadsRule WRITE setAutomaticMediaDownloadsRule
@@ -66,6 +68,8 @@ class AccountSettings : public QObject
     Q_PROPERTY(AccountSettings::GeoLocationMapService geoLocationMapService READ geoLocationMapService WRITE setGeoLocationMapService NOTIFY
                    geoLocationMapServiceChanged)
 
+    Q_PROPERTY(QString loginUriString READ loginUriString NOTIFY passwordChanged)
+
 public:
     /**
      * State specifying in which way a password is displayed.
@@ -129,12 +133,15 @@ public:
         QXmppConfiguration::StreamSecurityMode tlsRequirement = QXmppConfiguration::TLSRequired;
         QString host;
         quint16 port = AUTO_DETECT_PORT;
+        bool plainAuthAllowed = false;
         QString name;
         bool enabled = true;
         QString latestMessageStanzaId;
         QDateTime latestMessageStanzaTimestamp;
         bool inBandRegistrationFeaturesSupported = false;
         qint64 httpUploadLimit = 0;
+        QList<QString> chatSupportAddresses;
+        QList<QString> groupChatSupportAddresses;
         PasswordVisibility passwordVisibility = PasswordVisibility::Visible;
         Encryption::Enum encryption = Encryption::Omemo2;
         AutomaticMediaDownloadsRule automaticMediaDownloadsRule = AutomaticMediaDownloadsRule::PresenceOnly;
@@ -181,6 +188,10 @@ public:
 
     quint16 autoDetectPort() const;
 
+    bool plainAuthAllowed() const;
+    void setPlainAuthAllowed(bool plainAuthAllowed);
+    Q_SIGNAL void plainAuthAllowedChanged();
+
     QString name() const;
     void setName(const QString &name);
     Q_SIGNAL void nameChanged();
@@ -201,6 +212,14 @@ public:
 
     QString httpUploadLimitText() const;
 
+    void setChatSupportAddresses(const QList<QString> &chatSupportAddresses);
+    QList<QString> chatSupportAddresses() const;
+    Q_SIGNAL void chatSupportAddressesChanged();
+
+    void setGroupChatSupportAddresses(const QList<QString> &groupChatSupportAddresses);
+    QList<QString> groupChatSupportAddresses() const;
+    Q_SIGNAL void groupChatSupportAddressesChanged();
+
     PasswordVisibility passwordVisibility() const;
     void setPasswordVisibility(PasswordVisibility passwordVisibility);
     Q_SIGNAL void passwordVisibilityChanged();
@@ -233,6 +252,8 @@ public:
 
     void storeTemporaryData();
 
+    QString loginUriString() const;
+
 private:
     void generateJidResource();
     void generateUserAgentDeviceId();
@@ -296,7 +317,6 @@ class Account : public QObject
     Q_PROPERTY(VCardController *vCardController READ vCardController CONSTANT)
     Q_PROPERTY(VersionController *versionController READ versionController CONSTANT)
 
-    Q_PROPERTY(AvatarCache *avatarCache READ avatarCache NOTIFY avatarCacheChanged)
     Q_PROPERTY(PresenceCache *presenceCache READ presenceCache CONSTANT)
 
 public:
@@ -331,8 +351,6 @@ public:
     VCardController *vCardController() const;
     VersionController *versionController() const;
 
-    AvatarCache *avatarCache() const;
-    Q_SIGNAL void avatarCacheChanged();
     PresenceCache *presenceCache() const;
 
     Q_INVOKABLE void enable();
@@ -380,14 +398,13 @@ private:
 
     Connection *const m_connection;
 
-    AvatarCache *const m_avatarCache;
     PresenceCache *const m_presenceCache;
 
     AtmController *const m_atmController;
     BlockingController *const m_blockingController;
     EncryptionController *const m_encryptionController;
-    FileSharingController *const m_fileSharingController;
     RosterController *const m_rosterController;
+    FileSharingController *const m_fileSharingController;
     MessageController *const m_messageController;
     GroupChatController *const m_groupChatController;
     NotificationController *const m_notificationController;
