@@ -16,8 +16,15 @@
 #include "PresenceCache.h"
 #include "RosterItemWatcher.h"
 
-class QTimer;
+class Account;
+class ChatHintModel;
+class EncryptionController;
 class EncryptionWatcher;
+class GroupChatController;
+class MessageController;
+class MessageModel;
+class NotificationController;
+class QTimer;
 
 class ChatState : public QObject
 {
@@ -41,40 +48,38 @@ class ChatController : public QObject
 {
     Q_OBJECT
 
-    Q_PROPERTY(QString accountJid READ accountJid NOTIFY accountJidChanged)
-    Q_PROPERTY(QString chatJid READ chatJid NOTIFY chatJidChanged)
+    Q_PROPERTY(Account *account READ account NOTIFY accountChanged)
+    Q_PROPERTY(QString jid READ jid NOTIFY jidChanged)
+
     Q_PROPERTY(const RosterItem &rosterItem READ rosterItem NOTIFY rosterItemChanged)
+
     Q_PROPERTY(EncryptionWatcher *accountEncryptionWatcher READ accountEncryptionWatcher CONSTANT)
     Q_PROPERTY(EncryptionWatcher *chatEncryptionWatcher READ chatEncryptionWatcher CONSTANT)
+
+    Q_PROPERTY(ChatHintModel *chatHintModel READ chatHintModel NOTIFY chatHintModelChanged)
+    Q_PROPERTY(MessageModel *messageModel READ messageModel NOTIFY messageModelChanged)
+
     Q_PROPERTY(bool isEncryptionEnabled READ isEncryptionEnabled NOTIFY isEncryptionEnabledChanged)
     Q_PROPERTY(Encryption::Enum encryption READ encryption WRITE setEncryption NOTIFY encryptionChanged)
+    Q_PROPERTY(Encryption::Enum activeEncryption READ activeEncryption NOTIFY encryptionChanged)
+    Q_PROPERTY(QList<QString> groupChatUserJids READ groupChatUserJids NOTIFY groupChatUserJidsChanged)
     Q_PROPERTY(QXmppMessage::State chatState READ chatState NOTIFY chatStateChanged)
+    Q_PROPERTY(QString messageBodyToForward READ messageBodyToForward WRITE setMessageBodyToForward NOTIFY messageBodyToForwardChanged)
 
 public:
-    static ChatController *instance();
-
     explicit ChatController(QObject *parent = nullptr);
     ~ChatController();
 
-    QString accountJid();
-    Q_SIGNAL void accountJidChanged(const QString &accountJid);
-
-    QString chatJid();
-    Q_SIGNAL void chatJidChanged(const QString &chatJid);
-
-    Q_INVOKABLE void setChat(const QString &accountJid, const QString &chatJid);
-    Q_INVOKABLE void resetChat();
+    Q_INVOKABLE void initialize(Account *account, const QString &jid);
+    Q_SIGNAL void aboutToChangeChat();
     Q_SIGNAL void chatChanged();
 
-    /**
-     * Determines whether a chat is the currently open chat.
-     *
-     * @param accountJid JID of the chat's account
-     * @param chatJid JID of the chat
-     *
-     * @return true if the chat is currently open, otherwise false
-     */
-    bool isChatCurrentChat(const QString &accountJid, const QString &chatJid) const;
+    Account *account() const;
+    Q_SIGNAL void accountChanged();
+
+    QString jid();
+    void setJid(const QString &jid);
+    Q_SIGNAL void jidChanged();
 
     const RosterItem &rosterItem() const;
     Q_SIGNAL void rosterItemChanged();
@@ -82,7 +87,11 @@ public:
     EncryptionWatcher *accountEncryptionWatcher() const;
     EncryptionWatcher *chatEncryptionWatcher() const;
 
-    void handleGroupChatUserJidsFetched();
+    ChatHintModel *chatHintModel() const;
+    Q_SIGNAL void chatHintModelChanged();
+
+    MessageModel *messageModel() const;
+    Q_SIGNAL void messageModelChanged();
 
     Encryption::Enum activeEncryption() const;
 
@@ -93,40 +102,62 @@ public:
     void setEncryption(Encryption::Enum encryption);
     Q_SIGNAL void encryptionChanged();
 
+    QList<QString> groupChatUserJids() const;
+    Q_SIGNAL void groupChatUserJidsChanged();
+
     Q_INVOKABLE void resetComposingChatState();
 
-    /**
-     * Returns the current chat state
-     */
     QXmppMessage::State chatState() const;
     Q_SIGNAL void chatStateChanged();
 
-    /**
-     * Sends the chat state notification
-     */
-    void sendChatState(QXmppMessage::State state);
     Q_INVOKABLE void sendChatState(ChatState::State state);
+    void sendChatState(QXmppMessage::State state);
 
-    void handleChatState(const QString &bareJid, QXmppMessage::State state);
+    QString messageBodyToForward() const;
+    void setMessageBodyToForward(const QString &messageBodyToForward);
+    Q_SIGNAL void messageBodyToForwardChanged();
 
 private:
+    void initializeEncryption();
     bool hasUsableEncryptionDevices() const;
-    void resetChat(const QString &accountJid, const QString &chatJid);
 
-    QString m_accountJid;
-    QString m_chatJid;
+    void initializeGroupChat();
+    void updateGroupChatUserJids(const QString &accountJid, const QString &groupChatJid);
+    void updateGroupChatEncryption();
+    void setGroupChatUserJids(const QList<QString> &groupChatUserJids);
+
+    void initializeChatStateHandling();
+    void resetChatStates();
+    void handleChatState(const QString &bareJid, QXmppMessage::State state);
+
+    void executeOnceConnected(std::function<void()> &&function);
+
+    Account *m_account = nullptr;
+    QString m_jid;
+
+    EncryptionController *m_encryptionController = nullptr;
+    GroupChatController *m_groupChatController = nullptr;
+    MessageController *m_messageController = nullptr;
+    NotificationController *m_notificationController = nullptr;
+
     RosterItemWatcher m_rosterItemWatcher;
     UserResourcesWatcher m_contactResourcesWatcher;
     EncryptionWatcher *const m_accountEncryptionWatcher;
     EncryptionWatcher *const m_chatEncryptionWatcher;
 
+    ChatHintModel *m_chatHintModel = nullptr;
+    MessageModel *m_messageModel = nullptr;
+
     QXmppMessage::State m_chatPartnerChatState = QXmppMessage::State::None;
     QXmppMessage::State m_ownChatState = QXmppMessage::State::None;
     QTimer *const m_composingTimer;
     QTimer *const m_stateTimeoutTimer;
     QTimer *const m_inactiveTimer;
-    QTimer *const m_chatPartnerChatStateTimeout;
+    QTimer *const m_chatPartnerChatStateTimer;
     QMap<QString, QXmppMessage::State> m_chatStateCache;
 
-    static ChatController *s_instance;
+    QList<QString> m_groupChatUserJids;
+    QString m_messageBodyToForward;
+
+    QList<QMetaObject::Connection> m_offlineConnections;
 };
