@@ -34,14 +34,13 @@
 #include <QXmppRosterManager.h>
 #include <QXmppSasl2UserAgent.h>
 #include <QXmppStreamError.h>
-#include <QXmppUploadRequestManager.h>
 #include <QXmppUtils.h>
 #include <QXmppVCardManager.h>
 #include <QXmppVersionManager.h>
 // Kaidan
 #include "Account.h"
 #include "AtmController.h"
-#include "DiscoveryManager.h"
+#include "DiscoveryController.h"
 #include "EncryptionController.h"
 #include "KaidanCoreLog.h"
 #include "MediaUtils.h"
@@ -61,9 +60,9 @@ ClientWorker::ClientWorker(AccountSettings *accountSettings, QObject *parent)
     m_accountMigrationManager = m_client->addNewExtension<QXmppAccountMigrationManager>();
     m_blockingManager = m_client->addNewExtension<QXmppBlockingManager>();
     m_client->addNewExtension<QXmppCarbonManagerV2>();
-    auto *discoveryManager = m_client->addNewExtension<QXmppDiscoveryManager>();
+    m_discoveryManager = m_client->addNewExtension<QXmppDiscoveryManager>();
     m_client->addNewExtension<QXmppEntityTimeManager>();
-    auto *uploadManager = m_client->addNewExtension<QXmppHttpUploadManager>(networkAccessManager);
+    m_uploadManager = m_client->addNewExtension<QXmppHttpUploadManager>(networkAccessManager);
     m_mamManager = m_client->addNewExtension<QXmppMamManager>();
     m_client->addNewExtension<QXmppPubSubManager>();
     m_movedManager = m_client->addNewExtension<QXmppMovedManager>();
@@ -73,17 +72,14 @@ ClientWorker::ClientWorker(AccountSettings *accountSettings, QObject *parent)
     m_messageReceiptManager = m_client->addNewExtension<QXmppMessageReceiptManager>();
     m_registrationManager = m_client->addNewExtension<QXmppRegistrationManager>();
     m_rosterManager = m_client->addNewExtension<QXmppRosterManager>(m_client);
-    m_uploadRequestManager = m_client->addNewExtension<QXmppUploadRequestManager>();
     m_vCardManager = m_client->addNewExtension<QXmppVCardManager>();
     m_versionManager = m_client->addNewExtension<QXmppVersionManager>();
     m_mixManager = m_client->addNewExtension<QXmppMixManager>();
 
-    new DiscoveryManager(m_client, discoveryManager, this);
-
     // file sharing manager
     m_fileSharingManager = m_client->addNewExtension<QXmppFileSharingManager>();
     m_fileSharingManager->setMetadataGenerator(MediaUtils::generateMetadata);
-    m_httpProvider = std::make_shared<QXmppHttpFileSharingProvider>(uploadManager, networkAccessManager);
+    m_httpProvider = std::make_shared<QXmppHttpFileSharingProvider>(m_uploadManager, networkAccessManager);
     m_encryptedProvider = std::make_shared<QXmppEncryptedFileSharingProvider>(m_fileSharingManager, m_httpProvider);
     m_fileSharingManager->registerProvider(m_httpProvider);
     m_fileSharingManager->registerProvider(m_encryptedProvider);
@@ -145,7 +141,9 @@ void ClientWorker::logIn()
                       config.setAutoAcceptSubscriptions(false);
 
                       // Servers using LDAP need SASL PLAIN authentication.
-                      config.setDisabledSaslMechanisms({});
+                      if (m_accountSettings->plainAuthAllowed()) {
+                          config.setDisabledSaslMechanisms({});
+                      }
 
                       return config;
                   })
@@ -241,7 +239,6 @@ void ClientWorker::logOut(bool isApplicationBeingClosed)
 
 void ClientWorker::onConnected()
 {
-    // no mutex needed, because this is called from updateClient()
     qCDebug(KAIDAN_CORE_LOG) << "Connected to server";
 
     // If there was an error before, notify about its absence.
@@ -276,7 +273,15 @@ void ClientWorker::onConnected()
 
             runOnThread(m_encryptionController, [this]() {
                 m_encryptionController->setUp().then(m_messageController, [this]() {
-                    m_messageController->sendPendingData();
+                    if (m_uploadManager->support() == QXmppHttpUploadManager::Support::Unknown) {
+                        connect(m_uploadManager,
+                                &QXmppHttpUploadManager::servicesChanged,
+                                m_messageController,
+                                &MessageController::sendPendingData,
+                                Qt::SingleShotConnection);
+                    } else {
+                        m_messageController->sendPendingData();
+                    }
                 });
             });
         });
@@ -305,7 +310,6 @@ void ClientWorker::onConnectionStateChanged(QXmppClient::State connectionState)
 
 void ClientWorker::onConnectionError(const QXmppError &error)
 {
-    // no mutex needed, because this is called from updateClient()
     qCDebug(KAIDAN_CORE_LOG) << "Connection error:" << error.description;
 
     if (const auto socketError = error.value<QAbstractSocket::SocketError>()) {
