@@ -20,9 +20,11 @@ class PresenceCache;
 class QXmppAccountMigrationManager;
 class QXmppAtmManager;
 class QXmppBlockingManager;
+class QXmppDiscoveryManager;
 class QXmppEncryptedFileSharingProvider;
 class QXmppFileSharingManager;
 class QXmppHttpFileSharingProvider;
+class QXmppHttpUploadManager;
 class QXmppMamManager;
 class QXmppMessageReceiptManager;
 class QXmppMixManager;
@@ -30,7 +32,6 @@ class QXmppMovedManager;
 class QXmppOmemoManager;
 class QXmppRegistrationManager;
 class QXmppRosterManager;
-class QXmppUploadRequestManager;
 class QXmppVCardManager;
 class QXmppVersionManager;
 class RegistrationController;
@@ -86,6 +87,11 @@ public:
         return m_blockingManager;
     }
 
+    QXmppDiscoveryManager *discoveryManager() const
+    {
+        return m_discoveryManager;
+    }
+
     QXmppFileSharingManager *fileSharingManager() const
     {
         return m_fileSharingManager;
@@ -126,9 +132,9 @@ public:
         return m_rosterManager;
     }
 
-    QXmppUploadRequestManager *uploadRequestManager() const
+    QXmppHttpUploadManager *uploadManager() const
     {
-        return m_uploadRequestManager;
+        return m_uploadManager;
     }
 
     QXmppVCardManager *vCardManager() const
@@ -229,6 +235,7 @@ private:
     QXmppAccountMigrationManager *m_accountMigrationManager;
     QXmppAtmManager *m_atmManager;
     QXmppBlockingManager *m_blockingManager;
+    QXmppDiscoveryManager *m_discoveryManager;
     QXmppFileSharingManager *m_fileSharingManager;
     QXmppMamManager *m_mamManager;
     QXmppMessageReceiptManager *m_messageReceiptManager;
@@ -237,7 +244,7 @@ private:
     QXmppOmemoManager *m_omemoManager;
     QXmppRegistrationManager *m_registrationManager;
     QXmppRosterManager *m_rosterManager;
-    QXmppUploadRequestManager *m_uploadRequestManager;
+    QXmppHttpUploadManager *m_uploadManager;
     QXmppVCardManager *m_vCardManager;
     QXmppVersionManager *m_versionManager;
 
