@@ -10,6 +10,7 @@
 
 // Qt
 #include <QLatin1String>
+#include <QObject>
 // Kaidan
 #include <GlobalsGen.h>
 
@@ -95,6 +96,7 @@ constexpr auto GENERATED_PASSWORD_LENGTH_UPPER_BOUND = 30;
  */
 #define PROVIDER_COMPLETION_LIST_FILE_PATH QStringLiteral(":/data/providers-completion.json")
 
+const auto NO_SELECTION_TEXT = QObject::tr("No selection");
 constexpr QStringView DEFAULT_LANGUAGE_CODE = u"EN";
 constexpr QStringView DEFAULT_COUNTRY_CODE = u"US";
 
@@ -104,10 +106,8 @@ constexpr QStringView DEFAULT_COUNTRY_CODE = u"US";
  */
 constexpr auto PROVIDER_LIST_MIN_PROVIDERS_FROM_COUNTRY = 2;
 
-/**
- * Name of the @c QQuickImageProvider for Bits of Binary.
- */
-#define BITS_OF_BINARY_IMAGE_PROVIDER_NAME "bits-of-binary"
+// Name of the provider for all images.
+const QString IMAGE_PROVIDER_NAME = QStringLiteral("image-provider");
 
 // Defines that a message is suitable for correction only if it is among the n most recent messages.
 constexpr int MAX_MESSAGE_CORRECTION_COUNT = 10;
