@@ -10,6 +10,7 @@
 // QXmpp
 #include <QXmppLogger.h>
 
+class AccountSettings;
 class QXmppClient;
 
 class LogHandler : public QObject
@@ -17,15 +18,7 @@ class LogHandler : public QObject
     Q_OBJECT
 
 public:
-    /**
-     * Default constructor
-     */
-    LogHandler(QXmppClient *client, bool enable, QObject *parent = nullptr);
-
-    /**
-     * Enable/disable logging to stdout (default: disabled)
-     */
-    void enableLogging(bool enable);
+    LogHandler(AccountSettings *accountSettings, QXmppClient *client, QObject *parent = nullptr);
 
     /**
      * Handles logging messages and processes them (currently only output
@@ -39,6 +32,5 @@ private:
      */
     static QString makeXmlPretty(QString inputXml);
 
-    QXmppClient *const m_client;
-    bool enabled = false;
+    AccountSettings *const m_accountSettings;
 };
