@@ -26,6 +26,8 @@
 #include <KIO/PreviewJob>
 // Kaidan
 #include "FutureUtils.h"
+#include "Globals.h"
+#include "ImageProvider.h"
 #include "KaidanCoreLog.h"
 #include "SystemUtils.h"
 
@@ -205,6 +207,13 @@ void MediaUtils::deleteFile(const QUrl &url)
     QFile::remove(url.toLocalFile());
 }
 
+void MediaUtils::deleteDownloadedFile(const QString &localFilePath)
+{
+    if (localFilePath.startsWith(SystemUtils::downloadDirectory())) {
+        QFile::remove(localFilePath);
+    }
+}
+
 QString MediaUtils::localFilePath(const QUrl &localFileUrl)
 {
     return localFileUrl.toLocalFile();
@@ -225,6 +234,11 @@ bool MediaUtils::imageValid(const QImage &image)
     return !image.isNull();
 }
 
+QMimeType MediaUtils::mimeType(const QByteArray &data)
+{
+    return s_mimeDB.mimeTypeForData(data);
+}
+
 QMimeType MediaUtils::mimeType(const QString &filePath)
 {
     if (filePath.isEmpty()) {
@@ -593,7 +607,7 @@ QFuture<std::shared_ptr<QXmppFileSharingManager::MetadataGeneratorResult>> Media
         return makeReadyFuture<std::shared_ptr<Result>>(std::move(result));
     }
 
-    QFutureInterface<std::shared_ptr<Result>> interface;
+    auto promise = std::make_shared<QPromise<std::shared_ptr<Result>>>();
 
     // create job
     auto *job =
@@ -616,46 +630,18 @@ QFuture<std::shared_ptr<QXmppFileSharingManager::MetadataGeneratorResult>> Media
         result->thumbnails.push_back(Thumnbnail{.width = uint32_t(image.size().width()),
                                                 .height = uint32_t(image.size().height()),
                                                 .data = thumbnailData,
-                                                .mimeType = QMimeDatabase().mimeTypeForData(thumbnailData)});
+                                                .mimeType = mimeType(thumbnailData)});
 
-        interface.reportResult(result);
-        interface.reportFinished();
+        promise->addResult(result);
+        promise->finish();
     });
 
-    connect(job, &KIO::PreviewJob::failed, [interface, result]() mutable {
-        interface.reportResult(result);
-        interface.reportFinished();
+    connect(job, &KIO::PreviewJob::failed, [promise, result]() mutable {
+        promise->addResult(result);
+        promise->finish();
     });
 
-    return interface.future();
-}
-
-QFuture<QByteArray> MediaUtils::generateThumbnail(const QUrl &localFileUrl, const QString &mimeTypeName, int edgePixelCount)
-{
-    QFutureInterface<QByteArray> interface;
-
-    KFileItemList items{KFileItem{
-        localFileUrl,
-        mimeTypeName,
-    }};
-
-    static auto allPlugins = KIO::PreviewJob::availablePlugins();
-
-    auto *job = new KIO::PreviewJob(items, QSize(edgePixelCount, edgePixelCount), &allPlugins);
-    job->setAutoDelete(true);
-
-    QObject::connect(job, &KIO::PreviewJob::gotPreview, [interface](auto, const QPixmap &preview) mutable {
-        reportFinishedResult(interface, MediaUtils::encodeImageThumbnail(preview));
-    });
-
-    QObject::connect(job, &KIO::PreviewJob::failed, [interface](const KFileItem &item) mutable {
-        qCDebug(KAIDAN_CORE_LOG) << "Could not generate a thumbnail for" << item.url();
-        reportFinishedResult(interface, {});
-    });
-
-    job->start();
-
-    return interface.future();
+    return promise->future();
 }
 
 #include "moc_MediaUtils.cpp"
