@@ -33,6 +33,7 @@
 #include "AccountDb.h"
 #include "Algorithms.h"
 #include "EncryptionController.h"
+#include "FileSharingController.h"
 #include "Globals.h"
 #include "GroupChatUser.h"
 #include "GroupChatUserDb.h"
@@ -48,12 +49,12 @@
 constexpr int MAM_BACKLOG_FETCH_COUNT = 40;
 
 using namespace std::placeholders;
-using std::ranges::find;
 
 MessageController::MessageController(AccountSettings *accountSettings,
                                      Connection *connection,
                                      EncryptionController *encryptionController,
                                      RosterController *rosterController,
+                                     FileSharingController *fileSharingController,
                                      QXmppClient *client,
                                      QXmppMamManager *mamManager,
                                      QXmppMessageReceiptManager *messageReceiptManager,
@@ -63,12 +64,15 @@ MessageController::MessageController(AccountSettings *accountSettings,
     , m_connection(connection)
     , m_encryptionController(encryptionController)
     , m_rosterController(rosterController)
+    , m_fileSharingController(fileSharingController)
     , m_client(client)
     , m_mamManager(mamManager)
     , m_messageReceiptManager(messageReceiptManager)
 {
     connect(RosterDb::instance(), &RosterDb::itemsReplaced, this, &MessageController::handleRosterReceived);
 
+    connect(m_fileSharingController, &FileSharingController::filesUploadedForPendingMessage, this, &MessageController::sendPendingMessageWithUploadedFiles);
+
     runOnThread(m_client, [this]() {
         connect(m_client, &QXmppClient::messageReceived, this, [this](const QXmppMessage &msg) {
             handleMessage(msg, MessageOrigin::Stream);
@@ -91,7 +95,8 @@ MessageController::MessageController(AccountSettings *accountSettings,
 
 QFuture<QXmpp::SendResult> MessageController::send(QXmppMessage &&message, Encryption::Enum encryption, const QList<QString> &encryptionJids)
 {
-    QFutureInterface<QXmpp::SendResult> interface(QFutureInterfaceBase::Started);
+    auto promise = std::make_shared<QPromise<QXmpp::SendResult>>();
+    promise->start();
 
     const auto recipientJid = message.to();
 
@@ -104,31 +109,31 @@ QFuture<QXmpp::SendResult> MessageController::send(QXmppMessage &&message, Encry
             message.addHint(QXmppMessage::Store);
         }
 
-        runOnThread(m_client, [this, interface, message]() mutable {
+        runOnThread(m_client, [this, promise, message]() mutable {
             m_client->send(std::move(message)).then(this, [=](QXmpp::SendResult &&result) mutable {
-                reportFinishedResult(interface, result);
+                reportFinishedResult(*promise, result);
             });
         });
     } else {
         if (encryptionJids.isEmpty()) {
-            runOnThread(m_client, [this, interface, message]() mutable {
+            runOnThread(m_client, [this, promise, message]() mutable {
                 m_client->sendSensitive(std::move(message)).then(this, [=](QXmpp::SendResult &&result) mutable {
-                    reportFinishedResult(interface, result);
+                    reportFinishedResult(*promise, result);
                 });
             });
         } else {
             QXmppSendStanzaParams params;
             params.setEncryptionJids(encryptionJids);
 
-            runOnThread(m_client, [this, interface, message, params]() mutable {
+            runOnThread(m_client, [this, promise, message, params]() mutable {
                 m_client->sendSensitive(std::move(message), params).then(this, [=](QXmpp::SendResult &&result) mutable {
-                    reportFinishedResult(interface, result);
+                    reportFinishedResult(*promise, result);
                 });
             });
         }
     }
 
-    return interface.future();
+    return promise->future();
 }
 
 void MessageController::sendMessageWithUndecidedEncryption(Message message)
@@ -254,69 +259,74 @@ void MessageController::updateMessageReactionsAfterSending(const QString &chatJi
 void MessageController::sendPendingMessage(Message message)
 {
     if (m_connection->state() == Enums::ConnectionState::StateConnected) {
-        auto sendMessage = [this](Message &&message, const QList<QString> &encryptionJids = {}) {
-            send(message.toQXmpp(), message.encryption, encryptionJids)
-                .then(this,
-                      [this,
-                       accountJid = m_accountSettings->jid(),
-                       chatJid = message.chatJid,
-                       messageId = message.id,
-                       fileFallbackMessages = message.fileFallbackMessages(),
-                       encryption = message.encryption,
-                       encryptionJids](QXmpp::SendResult result) mutable {
-                          if (const auto error = std::get_if<QXmppError>(&result)) {
-                              qCWarning(KAIDAN_CORE_LOG) << "Could not send message:" << error->description;
+        if (!std::ranges::all_of(message.files, [](const File &file) {
+                return file.transferState == File::TransferState::Done;
+            })) {
+            m_fileSharingController->sendPendingFiles(message.chatJid, message.id, message.files, message.encryption != Encryption::NoEncryption);
+            return;
+        }
 
-                              // The error message of the message is saved untranslated. To make
-                              // translation work in the UI, the tr() call of the passive
-                              // notification must contain exactly the same string.
-                              Q_EMIT MainController::instance()->passiveNotificationRequested(tr("Message could not be sent."));
-                              MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
-                                  msg.deliveryState = Enums::DeliveryState::Error;
-                                  msg.errorText = QStringLiteral("Message could not be sent.");
-                              });
-                          } else {
-                              MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
-                                  msg.deliveryState = Enums::DeliveryState::Sent;
-                                  msg.errorText.clear();
-                              });
+        sendPendingMessageWithUploadedFiles(message);
+    }
+}
 
-                              for (auto &fileFallbackMessage : fileFallbackMessages) {
-                                  // TODO: Track sending of fallback messages individually
-                                  // Needed for the case when the success differs between the main message
-                                  // and the fallback messages.
-                                  send(std::move(fileFallbackMessage), encryption, encryptionJids);
-                              }
+void MessageController::sendPendingMessageWithUploadedFiles(Message message)
+{
+    auto sendMessage = [this](Message &&message, const QList<QString> &encryptionJids = {}) {
+        send(message.toQXmpp(), message.encryption, encryptionJids)
+            .then(this,
+                  [this,
+                   accountJid = m_accountSettings->jid(),
+                   chatJid = message.chatJid,
+                   messageId = message.id,
+                   fileFallbackMessages = message.fileFallbackMessages(),
+                   encryption = message.encryption,
+                   encryptionJids](QXmpp::SendResult result) mutable {
+                      if (const auto error = std::get_if<QXmppError>(&result)) {
+                          qCWarning(KAIDAN_CORE_LOG) << "Could not send message:" << error->description;
+                          Q_EMIT MainController::instance()->passiveNotificationRequested(tr("Message could not be sent"));
+
+                          MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
+                              msg.deliveryState = Enums::DeliveryState::Error;
+                              msg.errorText = tr("Message could not be sent.");
+                          });
+                      } else {
+                          MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
+                              msg.deliveryState = Enums::DeliveryState::Sent;
+                              msg.errorText.clear();
+                          });
+
+                          for (auto &fileFallbackMessage : fileFallbackMessages) {
+                              // TODO: Track sending of fallback messages individually
+                              // Needed for the case when the success differs between the main message
+                              // and the fallback messages.
+                              send(std::move(fileFallbackMessage), encryption, encryptionJids);
                           }
-                      });
-        };
+                      }
+                  });
+    };
 
-        // if the message is a pending edition of the existing in the history message
-        // I need to send it with the most recent stamp
-        // for that I'm gonna copy that message and update in the copy just the stamp
-        if (!message.replaceId.isEmpty()) {
-            message.timestamp = QDateTime::currentDateTimeUtc();
-        }
+    if (!message.replaceId.isEmpty()) {
+        message.timestamp = QDateTime::currentDateTimeUtc();
+    }
 
-        const auto isGroupChatMessage = message.isGroupChatMessage();
+    const auto isGroupChatMessage = message.isGroupChatMessage();
 
-        message.receiptRequested = !isGroupChatMessage;
+    message.receiptRequested = !isGroupChatMessage;
 
-        if (isGroupChatMessage) {
-            GroupChatUserDb::instance()
-                ->userJids(message.accountJid, message.chatJid)
-                .then(this, [sendMessage, message](QList<QString> &&encryptionJids) mutable {
-                    sendMessage(std::move(message), encryptionJids);
-                });
-        } else {
-            sendMessage(std::move(message));
-        }
+    if (isGroupChatMessage) {
+        GroupChatUserDb::instance()->userJids(message.accountJid, message.chatJid).then(this, [sendMessage, message](QList<QString> &&encryptionJids) mutable {
+            sendMessage(std::move(message), encryptionJids);
+        });
+    } else {
+        sendMessage(std::move(message));
     }
 }
 
 QFuture<bool> MessageController::retrieveBacklogMessages(const QString &jid, bool isGroupChat, const QString &oldestMessageStanzaId)
 {
-    QFutureInterface<bool> interface(QFutureInterfaceBase::Started);
+    auto promise = std::make_shared<QPromise<bool>>();
+    promise->start();
 
     QXmppResultSetQuery queryLimit;
     queryLimit.setMax(MAM_BACKLOG_FETCH_COUNT);
@@ -329,10 +339,10 @@ QFuture<bool> MessageController::retrieveBacklogMessages(const QString &jid, boo
             return std::pair{m_mamManager->retrieveMessages(isGroupChat ? jid : QString{}, {}, isGroupChat ? QString{} : jid, {}, {}, queryLimit), this};
         },
         this,
-        [this, interface, jid](QXmppMamManager::RetrieveResult &&result) mutable {
+        [this, promise, jid](QXmppMamManager::RetrieveResult &&result) mutable {
             if (auto error = std::get_if<QXmppError>(&result)) {
                 qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve backlog messages:" << error->description;
-                reportFinishedResult(interface, false);
+                reportFinishedResult(*promise, false);
             } else {
                 const auto retrievedMessages = std::get<QXmppMamManager::RetrievedMessages>(std::move(result));
                 const auto messages = retrievedMessages.messages;
@@ -341,11 +351,11 @@ QFuture<bool> MessageController::retrieveBacklogMessages(const QString &jid, boo
                     handleMessage(message, MessageOrigin::MamBacklog);
                 }
 
-                reportFinishedResult(interface, retrievedMessages.result.complete());
+                reportFinishedResult(*promise, retrievedMessages.result.complete());
             }
         });
 
-    return interface.future();
+    return promise->future();
 }
 
 void MessageController::sendReadMarker(const QString &chatJid, const QString &messageId, Encryption::Enum encryption, const QList<QString> &encryptionJids)
@@ -630,7 +640,7 @@ void MessageController::handleMessage(const QXmppMessage &msg, MessageOrigin ori
     message.spoilerHint = msg.spoilerHint();
 
     // file sharing messages for backwards-compatibility are ignored
-    if (find(msg.fallbackMarkers(), XMLNS_SFS, &QXmppFallback::forNamespace) != msg.fallbackMarkers().end() && msg.sharedFiles().empty()) {
+    if (std::ranges::contains(msg.fallbackMarkers(), XMLNS_SFS, &QXmppFallback::forNamespace) && msg.sharedFiles().empty()) {
         return;
     }
 
