@@ -37,6 +37,12 @@ public:
         int queryIndex = -1;
     };
 
+    struct DownloadableFile {
+        QString chatJid;
+        QString messageId;
+        File file;
+    };
+
     explicit MessageDb(QObject *parent = nullptr);
     ~MessageDb() override;
 
@@ -80,7 +86,7 @@ public:
      *
      * @param accountJid bare JID of the user's account
      *
-     * @return the fetched messages
+     * @return the fetched files
      */
     QFuture<QList<File>> fetchFiles(const QString &accountJid);
 
@@ -90,7 +96,7 @@ public:
      * @param accountJid bare JID of the user's account
      * @param chatJid bare JID of the chat
      *
-     * @return the fetched messages
+     * @return the fetched files
      */
     QFuture<QList<File>> fetchFiles(const QString &accountJid, const QString &chatJid);
 
@@ -99,7 +105,7 @@ public:
      *
      * @param accountJid bare JID of the user's account
      *
-     * @return the fetched messages
+     * @return the fetched files
      */
     QFuture<QList<File>> fetchDownloadedFiles(const QString &accountJid);
 
@@ -109,10 +115,19 @@ public:
      * @param accountJid bare JID of the user's account
      * @param chatJid bare JID of the chat
      *
-     * @return the fetched messages
+     * @return the fetched files
      */
     QFuture<QList<File>> fetchDownloadedFiles(const QString &accountJid, const QString &chatJid);
 
+    /**
+     * Fetches automatically downloadable shared media of an account from the database.
+     *
+     * @param accountJid bare JID of the user's account
+     *
+     * @return the fetched files
+     */
+    QFuture<QList<DownloadableFile>> fetchAutomaticallyDownloadableFiles(const QString &accountJid);
+
     /**
      * Fetches messages until the first message of a specific chat.
      *
@@ -264,7 +279,7 @@ public:
     /**
      * Adds additional file sources to a file.
      *
-     * The DB file IDs in the sources are overriden and may e.g. be set to 0 if unknown.
+     * The DB file IDs in the sources are overridden and may e.g. be set to 0 if unknown.
      * The whole message is fetched and messageUpdated() is emitted.
      */
     QFuture<void> attachFileSources(const QString &accountJid,
@@ -308,7 +323,11 @@ private:
     void _setFileHashes(const QList<FileHash> &fileHashes);
     void _setHttpSources(const QList<HttpSource> &sources);
     void _setEncryptedSources(const QList<EncryptedSource> &sources);
+    void _removeFiles(const QString &accountJid);
+    void _removeFiles(const QString &accountJid, const QString &chatJid);
+    void _removeFiles(const QString &accountJid, const QString &chatJid, const QString &messageId);
     void _removeFiles(const QList<qint64> &fileIds);
+    void _removeFiles(const QString &statement, const QueryBindValues &bindValues);
     void _removeFileHashes(const QList<qint64> &fileIds);
     void _removeHttpSources(const QList<qint64> &fileIds);
     void _removeEncryptedSources(const QList<qint64> &fileIds);
@@ -323,9 +342,15 @@ private:
     QList<EncryptedSource> _fetchEncryptedSource(qint64 fileId);
 
     void _fetchAdditionalData(QList<Message> &messages);
+
     void _fetchReactions(QList<Message> &messages);
+    void _removeReactions(const QString &accountJid);
+    void _removeReactions(const QString &accountJid, const QString &chatJid);
+    void _removeReactions(const QString &accountJid, const QString &chatJid, const QString &messageId);
+
     void _fetchGroupChatUsers(QList<Message> &messages);
     void _fetchGroupChatUser(Message &message);
+
     void _fetchTrustLevels(QList<Message> &messages);
     void _fetchTrustLevel(Message &messages);
 
