@@ -113,6 +113,8 @@ public:
     Q_SIGNAL void messageFetchingFinished();
     Q_INVOKABLE bool canFetchMore(const QModelIndex &parent) const override;
 
+    Q_INVOKABLE void resendMessage(int index);
+
     Q_INVOKABLE void handleMessageRead(int readMessageIndex);
     Q_INVOKABLE int firstUnreadContactMessageIndex() const;
     Q_INVOKABLE void setMessageMarked(int index, bool marked);
@@ -147,6 +149,8 @@ public:
 
     Q_INVOKABLE bool canCorrectMessage(int index) const;
 
+    Q_INVOKABLE void deleteFile(int index, const File &file);
+
     /**
      * Removes a message locally.
      *
@@ -228,8 +232,6 @@ private:
      */
     void removeMessages(const QString &accountJid, const QString &chatJid = {});
 
-    void addVideoThumbnails(const Message &message);
-
     void updateLastReadOwnMessageId();
     void updateFirstUnreadContactMessageIndex();
 
