@@ -12,9 +12,9 @@
 class MessageReactionModel : public QAbstractListModel
 {
     Q_OBJECT
-    Q_PROPERTY(QString accountJid MEMBER accountJid)
-    Q_PROPERTY(QString chatJid MEMBER chatJid)
-    Q_PROPERTY(QList<DetailedMessageReaction> reactions MEMBER reactions WRITE setReactions)
+    Q_PROPERTY(QString accountJid MEMBER m_accountJid WRITE setAccountJid)
+    Q_PROPERTY(QString chatJid MEMBER m_chatJid WRITE setChatJid)
+    Q_PROPERTY(QList<DetailedMessageReaction> reactions MEMBER m_reactions WRITE setReactions)
 
 public:
     enum class Role {
@@ -30,10 +30,14 @@ public:
     [[nodiscard]] QHash<int, QByteArray> roleNames() const override;
     [[nodiscard]] QVariant data(const QModelIndex &index, int role) const override;
 
+    void setAccountJid(const QString &accountJid);
+    void setChatJid(const QString &chatJid);
     void setReactions(const QList<DetailedMessageReaction> &reactions);
 
 private:
-    QString accountJid;
-    QString chatJid;
-    QList<DetailedMessageReaction> reactions;
+    void updateGroupChatUserData();
+
+    QString m_accountJid;
+    QString m_chatJid;
+    QList<DetailedMessageReaction> m_reactions;
 };
