@@ -12,8 +12,6 @@
 #include <QGeoCoordinate>
 #include <QGuiApplication>
 #include <QImage>
-#include <QMimeData>
-#include <QMimeDatabase>
 #include <QRegularExpression>
 #include <QStandardPaths>
 // QXmpp
@@ -147,18 +145,32 @@ QUrl QmlUtils::providerDetailsUrl(const QString &providerJid)
     return QUrl(PROVIDER_DETAILS_URL.arg(providerJid));
 }
 
-QUrl QmlUtils::invitationUrl(const QString &uri)
+QString QmlUtils::invitationUrlString(const QString &uri)
 {
     // "uri.mid(5)" removes "xmpp:".
-    return QUrl(QStringLiteral(INVITATION_URL) + uri.mid(5));
+    return QStringLiteral(INVITATION_URL) + uri.mid(5);
+}
+
+QString QmlUtils::jid(const QString &uriString)
+{
+    if (const auto uriParsingResult = QXmppUri::fromString(uriString); std::holds_alternative<QXmppUri>(uriParsingResult)) {
+        return std::get<QXmppUri>(uriParsingResult).jid();
+    }
+
+    return {};
 }
 
 QUrl QmlUtils::groupChatUri(const QString &groupChatJid)
+{
+    return QUrl(groupChatUriString(groupChatJid));
+}
+
+QString QmlUtils::groupChatUriString(const QString &groupChatJid)
 {
     QXmppUri uri;
     uri.setJid(groupChatJid);
     uri.setQuery(QXmpp::Uri::Join());
-    return QUrl(uri.toString());
+    return uri.toString();
 }
 
 bool QmlUtils::validateEncryptionKeyId(const QString &keyId)
