@@ -11,8 +11,6 @@
 #include <QUrl>
 // QXmpp
 #include <QXmppMessage.h>
-// Kaidan
-#include "ClientWorker.h"
 
 class QGeoCoordinate;
 
@@ -49,13 +47,6 @@ public:
 
     Q_INVOKABLE static QString systemCountryCode();
 
-    /**
-     * Returns an error message for a connection error.
-     *
-     * @param error error for which an error message should be returned
-     */
-    Q_INVOKABLE static QString connectionErrorMessage(ClientWorker::ConnectionError error);
-
     /**
      * Returns a URL to a given resource file name
      *
@@ -108,9 +99,16 @@ public:
     Q_INVOKABLE static QUrl providerDetailsUrl(const QString &providerJid);
 
     /**
-     * Returns an invitation URL for the given XMPP URI.
+     * Returns an invitation URL as a string for the given XMPP URI.
      */
-    Q_INVOKABLE static QUrl invitationUrl(const QString &uri);
+    Q_INVOKABLE static QString invitationUrlString(const QString &uri);
+
+    /**
+     * Returns the JID of an XMPP URI.
+     *
+     * If the URI is not valid, a default-constructed string is returned.
+     */
+    Q_INVOKABLE static QString jid(const QString &uriString);
 
     /**
      * Returns an XMPP URI for opening a group chat.
@@ -119,6 +117,13 @@ public:
      */
     Q_INVOKABLE static QUrl groupChatUri(const QString &groupChatJid);
 
+    /**
+     * Returns an XMPP URI as a string for opening a group chat.
+     *
+     * @param groupChatJid JID of the group chat
+     */
+    Q_INVOKABLE static QString groupChatUriString(const QString &groupChatJid);
+
     /**
      * Validates the ID of an encryption key.
      *
@@ -180,11 +185,4 @@ public:
 
     Q_INVOKABLE static QString geoUri(const QGeoCoordinate &geoCoordinate);
     Q_INVOKABLE static QGeoCoordinate geoCoordinate(const QString &geoUri);
-
-    /**
-     * Opens a geo location as preferred by the user.
-     *
-     * @return whether the location should be opened within Kaidan
-     */
-    Q_INVOKABLE static bool openGeoLocation(const QGeoCoordinate &geoCoordinate);
 };
