@@ -83,7 +83,11 @@ RosterController::RosterController(AccountSettings *accountSettings,
         const auto accountJid = m_accountSettings->jid();
         MessageDb::instance()->removeMessages(accountJid, jid);
         RosterDb::instance()->removeItem(accountJid, jid);
-        m_encryptionController->removeContactDevices(jid);
+
+        // Do not remove own devices in case the notes chat is removed.
+        if (jid != accountJid) {
+            m_encryptionController->removeContactDevices(jid);
+        }
     });
 
     connect(m_manager, &QXmppRosterManager::subscriptionRequestReceived, this, &RosterController::handleSubscriptionRequest);
@@ -97,7 +101,7 @@ RosterController::RosterController(AccountSettings *accountSettings,
     connect(RosterModel::instance(), &RosterModel::groupsChanged, this, &RosterController::groupsChanged);
 }
 
-RosterController::ContactAdditionWithUriResult RosterController::addContactWithUri(const QString &uriString)
+RosterController::ContactAdditionWithUriResult RosterController::addContactWithUri(const QString &uriString, const QString &name, const QString &message)
 {
     if (const auto uriParsingResult = QXmppUri::fromString(uriString); std::holds_alternative<QXmppUri>(uriParsingResult)) {
         const auto uri = std::get<QXmppUri>(uriParsingResult);
@@ -112,7 +116,7 @@ RosterController::ContactAdditionWithUriResult RosterController::addContactWithU
             return ContactAdditionWithUriResult::ContactExists;
         }
 
-        addContact(jid);
+        addContact(jid, name, message);
 
         return ContactAdditionWithUriResult::AddingContact;
     }
@@ -131,7 +135,7 @@ void RosterController::addContact(const QString &jid, const QString &name, const
             m_pendingAutomaticInitialAdditionJids.append(jid);
         }
 
-        // Do not try to add the own JID to the roster mutiple times if the server does not support it.
+        // Do not try to add the own JID to the roster multiple times if the server does not support it.
         if (const auto ownJidBeingAdded = jid == m_accountSettings->jid(); ownJidBeingAdded && !m_addingOwnJidToRosterAllowed) {
             if (!automaticInitialAddition) {
                 Q_EMIT contactAdditionFailed(jid);
