@@ -27,6 +27,15 @@ RosterItem::RosterItem(const QString &accountJid, const QXmppRosterIq::Item &ite
 
 QString RosterItem::displayName() const
 {
+    if (isProviderChat()) {
+        return QObject::tr("Provider");
+    }
+
+    // Consider the roster item as a chat with oneself if its JID matches the own JID.
+    if (jid == accountJid) {
+        return QObject::tr("Notes");
+    }
+
     if (name.isEmpty()) {
         // Return the JID if the roster item is created by a RosterItemWatcher while not being an
         // actual item in the roster.
@@ -34,18 +43,13 @@ QString RosterItem::displayName() const
             return jid;
         }
 
-        // Consider the roster item as a chat with oneself if its JID matches the own JID.
-        if (jid == accountJid) {
-            return QObject::tr("Notes");
-        }
-
         if (!groupChatName.isEmpty()) {
             return groupChatName;
         }
 
         const auto username = QXmppUtils::jidToUser(jid);
 
-        // Return the domain in case of a server as a roster item (for service announcements).
+        // Return the domain in case of a server as a roster item.
         if (username.isEmpty()) {
             return jid;
         }
@@ -66,6 +70,16 @@ bool RosterItem::isReceivingPresence() const
     return subscription == QXmppRosterIq::Item::From || subscription == QXmppRosterIq::Item::Both;
 }
 
+bool RosterItem::isNotesChat() const
+{
+    return accountJid == jid;
+}
+
+bool RosterItem::isProviderChat() const
+{
+    return QXmppUtils::jidToUser(jid).isEmpty() && QXmppUtils::jidToDomain(jid) == QXmppUtils::jidToDomain(accountJid);
+}
+
 bool RosterItem::isGroupChat() const
 {
     return !groupChatParticipantId.isEmpty();
@@ -124,72 +138,57 @@ RosterItem::EffectiveNotificationRule RosterItem::effectiveNotificationRule() co
     Q_UNREACHABLE();
 }
 
-bool RosterItem::operator<(const RosterItem &other) const
+bool RosterItem::automaticDownloadsEnabled() const
 {
-    if (pinningPosition == -1 && other.pinningPosition == -1) {
-        if (lastMessageDateTime != other.lastMessageDateTime) {
-            return lastMessageDateTime > other.lastMessageDateTime;
+    switch (automaticMediaDownloadsRule) {
+    case RosterItem::AutomaticMediaDownloadsRule::Account:
+        switch (AccountController::instance()->account(accountJid)->settings()->automaticMediaDownloadsRule()) {
+        case AccountSettings::AutomaticMediaDownloadsRule::Never:
+            return false;
+        case AccountSettings::AutomaticMediaDownloadsRule::PresenceOnly:
+            return isReceivingPresence() || isNotesChat();
+        case AccountSettings::AutomaticMediaDownloadsRule::Always:
+            return true;
         }
-
-        if (displayName() != other.displayName()) {
-            return displayName() < other.displayName();
-        }
-
-        return accountJid < other.accountJid;
+        Q_UNREACHABLE();
+    case RosterItem::AutomaticMediaDownloadsRule::Never:
+        return false;
+    case RosterItem::AutomaticMediaDownloadsRule::Always:
+        return true;
     }
 
-    return pinningPosition > other.pinningPosition;
+    Q_UNREACHABLE();
 }
 
-bool RosterItem::operator>(const RosterItem &other) const
+std::strong_ordering RosterItem::operator<=>(const RosterItem &other) const
 {
-    if (pinningPosition == -1 && other.pinningPosition == -1) {
-        if (lastMessageDateTime != other.lastMessageDateTime) {
-            return lastMessageDateTime < other.lastMessageDateTime;
-        }
-
-        if (displayName() != other.displayName()) {
-            return displayName() > other.displayName();
-        }
-
-        return accountJid > other.accountJid;
+    // If both items are un-/pinned and one of them has a draft message, the one with the draft message is above the other one.
+    if (((pinningPosition != -1) == (other.pinningPosition != -1))
+        && ((lastMessageDeliveryState == Enums::DeliveryState::Draft) != (other.lastMessageDeliveryState == Enums::DeliveryState::Draft))) {
+        return lastMessageDeliveryState == Enums::DeliveryState::Draft ? std::strong_ordering::less : std::strong_ordering::greater;
     }
 
-    return pinningPosition < other.pinningPosition;
-}
-
-bool RosterItem::operator<=(const RosterItem &other) const
-{
-    if (pinningPosition == -1 && other.pinningPosition == -1) {
-        if (lastMessageDateTime != other.lastMessageDateTime) {
-            return lastMessageDateTime >= other.lastMessageDateTime;
-        }
-
-        if (displayName() != other.displayName()) {
-            return displayName() <= other.displayName();
-        }
-
-        return accountJid <= other.accountJid;
+    // If either item is pinned (pinningPosition != -1), compare based on that.
+    // An item with a higher pinningPosition is above an item with a lower one.
+    if (pinningPosition != -1 || other.pinningPosition != -1) {
+        return other.pinningPosition <=> pinningPosition;
     }
 
-    return pinningPosition >= other.pinningPosition;
-}
-
-bool RosterItem::operator>=(const RosterItem &other) const
-{
-    if (pinningPosition == -1 && other.pinningPosition == -1) {
-        if (lastMessageDateTime != other.lastMessageDateTime) {
-            return lastMessageDateTime <= other.lastMessageDateTime;
-        }
-
-        if (displayName() != other.displayName()) {
-            return displayName() >= other.displayName();
-        }
-
-        return accountJid >= other.accountJid;
+    // If neither item is pinned, compare based on the last message date/time.
+    // An item with a more recent last message date/time is above an item with an older one.
+    if (lastMessageDateTime != other.lastMessageDateTime) {
+        return (lastMessageDateTime < other.lastMessageDateTime) ? std::strong_ordering::greater : std::strong_ordering::less;
     }
 
-    return pinningPosition <= other.pinningPosition;
+    if (displayName() != other.displayName()) {
+        return (displayName() < other.displayName()) ? std::strong_ordering::less : std::strong_ordering::greater;
+    }
+
+    if (accountJid != other.accountJid) {
+        return (accountJid < other.accountJid) ? std::strong_ordering::less : std::strong_ordering::greater;
+    }
+
+    return std::strong_ordering::equal;
 }
 
 #include "moc_RosterItem.cpp"
