@@ -8,6 +8,8 @@
 
 #pragma once
 
+// std
+#include <compare>
 // Qt
 #include <QDateTime>
 #include <QXmppRosterIq.h>
@@ -27,6 +29,7 @@ struct RosterItem {
     Q_PROPERTY(QString displayName READ displayName CONSTANT)
     Q_PROPERTY(bool sendingPresence READ isSendingPresence CONSTANT)
     Q_PROPERTY(bool receivingPresence READ isReceivingPresence CONSTANT)
+    Q_PROPERTY(bool isProviderChat READ isProviderChat CONSTANT)
     Q_PROPERTY(bool isGroupChat READ isGroupChat CONSTANT)
     Q_PROPERTY(bool isPublicGroupChat READ isPublicGroupChat CONSTANT)
     Q_PROPERTY(bool isDeletedGroupChat READ isDeletedGroupChat CONSTANT)
@@ -87,19 +90,27 @@ public:
     bool isSendingPresence() const;
     bool isReceivingPresence() const;
 
+    /**
+     * Returns whether the item is a chat with oneself.
+     */
+    bool isNotesChat() const;
+
+    /**
+     * Returns whether the item is a chat with the own service provider.
+     *
+     * Such a chat is often used for receiving a welcome message after account creation or for service announcements.
+     */
+    bool isProviderChat() const;
+
     bool isGroupChat() const;
     bool isPublicGroupChat() const;
     bool isDeletedGroupChat() const;
 
     EffectiveNotificationRule effectiveNotificationRule() const;
+    bool automaticDownloadsEnabled() const;
 
     bool operator==(const RosterItem &other) const = default;
-    bool operator!=(const RosterItem &other) const = default;
-
-    bool operator<(const RosterItem &other) const;
-    bool operator>(const RosterItem &other) const;
-    bool operator<=(const RosterItem &other) const;
-    bool operator>=(const RosterItem &other) const;
+    std::strong_ordering operator<=>(const RosterItem &other) const;
 
     // JID of the account that has this item.
     QString accountJid;
