@@ -45,11 +45,13 @@
 #if __has_include("KWindowSystem")
 #include <KWindowSystem>
 #endif
+#if defined(Q_OS_WINDOWS)
+#include <KIconTheme>
+#endif
 // KDAB
 #include <kdsingleapplication.h>
 // QXmpp
 #include <QXmppClient.h>
-#include <QXmppDiscoveryIq.h>
 #include <QXmppMixInfoItem.h>
 #include <QXmppMixParticipantItem.h>
 #include <QXmppRegisterIq.h>
@@ -61,20 +63,19 @@
 #include "Account.h"
 #include "AccountController.h"
 #include "AccountMigrationController.h"
-#include "AccountQrCodeGenerator.h"
+#include "AccountTrustMessageUriGenerator.h"
 #include "AtmController.h"
 #include "AuthenticatableEncryptionKeyModel.h"
 #include "AuthenticatedEncryptionKeyModel.h"
 #include "AvatarCache.h"
-#include "BitsOfBinaryImageProvider.h"
 #include "Blocking.h"
 #include "ChatController.h"
 #include "ChatHintModel.h"
-#include "ContactQrCodeGenerator.h"
+#include "ContactTrustMessageUriGenerator.h"
 #include "CredentialsGenerator.h"
 #include "CredentialsValidator.h"
 #include "DataFormModel.h"
-#include "DiscoveryManager.h"
+#include "DiscoveryController.h"
 #include "EmojiModel.h"
 #include "Encryption.h"
 #include "EncryptionController.h"
@@ -87,7 +88,6 @@
 #include "GlobalsGen.h"
 #include "GroupChatController.h"
 #include "GroupChatInviteeFilterModel.h"
-#include "GroupChatQrCodeGenerator.h"
 #include "GroupChatUser.h"
 #include "GroupChatUserFilterModel.h"
 #include "GroupChatUserKeyAuthenticationFilterModel.h"
@@ -95,16 +95,17 @@
 #include "GuiStyle.h"
 #include "HostCompletionModel.h"
 #include "HostCompletionProxyModel.h"
+#include "ImageProvider.h"
 #include "KaidanLog.h"
-#include "LoginQrCodeGenerator.h"
 #include "MainController.h"
 #include "MediaUtils.h"
 #include "Message.h"
 #include "MessageComposition.h"
 #include "MessageModel.h"
 #include "MessageReactionModel.h"
-#include "ProviderListItem.h"
-#include "ProviderListModel.h"
+#include "Provider.h"
+#include "ProviderFilterModel.h"
+#include "ProviderModel.h"
 #include "PublicGroupChatModel.h"
 #include "PublicGroupChatProxyModel.h"
 #include "PublicGroupChatSearchController.h"
@@ -129,7 +130,6 @@ Q_DECLARE_METATYPE(Qt::ApplicationState)
 
 Q_DECLARE_METATYPE(QXmppClient::State)
 Q_DECLARE_METATYPE(QXmppMessage::State)
-Q_DECLARE_METATYPE(QXmppDiscoveryIq)
 Q_DECLARE_METATYPE(QXmppPresence)
 Q_DECLARE_METATYPE(QXmppStanza::Error)
 Q_DECLARE_METATYPE(QXmppResultSetReply)
@@ -260,6 +260,10 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     Q_INIT_RESOURCE(images);
 #endif
 
+#if defined(Q_OS_WINDOWS)
+    KIconTheme::initTheme();
+#endif
+
     // name, display name, description
     QGuiApplication::setApplicationName(QStringLiteral(APPLICATION_NAME));
     QGuiApplication::setApplicationDisplayName(QStringLiteral(APPLICATION_DISPLAY_NAME));
@@ -298,7 +302,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     QApplication app(argc, argv);
 
     // register qMetaTypes
-    qRegisterMetaType<ProviderListItem>();
+    qRegisterMetaType<Provider>();
     qRegisterMetaType<RosterItem>();
     qRegisterMetaType<RosterItemWatcher *>();
     qRegisterMetaType<RosterModel *>();
@@ -306,7 +310,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<Message>();
     qRegisterMetaType<MessageModel *>();
     qRegisterMetaType<ChatHintModel *>();
-    qRegisterMetaType<DiscoveryManager *>();
+    qRegisterMetaType<DiscoveryController *>();
     qRegisterMetaType<VCardController *>();
     qRegisterMetaType<VersionController *>();
     qRegisterMetaType<RegistrationController *>();
@@ -350,7 +354,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<Presence::Availability>();
     qRegisterMetaType<Enums::DeliveryState>();
     qRegisterMetaType<MessageOrigin>();
-    qRegisterMetaType<ProviderListModel::Role>();
+    qRegisterMetaType<ProviderModel::Role>();
     qRegisterMetaType<ChatState::State>();
     qRegisterMetaType<Encryption>();
     qRegisterMetaType<MessageReactionDeliveryState>();
@@ -370,7 +374,6 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<QXmppResultSetReply>();
     qRegisterMetaType<QXmppMessage>();
     qRegisterMetaType<QXmppPresence>();
-    qRegisterMetaType<QXmppDiscoveryIq>();
     qRegisterMetaType<QHash<QString, QHash<QByteArray, QXmpp::TrustLevel>>>();
     qRegisterMetaType<QXmppMixInfoItem>();
     qRegisterMetaType<QXmppMixParticipantItem>();
@@ -475,7 +478,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 
     QQmlApplicationEngine engine;
 
-    engine.addImageProvider(QLatin1String(BITS_OF_BINARY_IMAGE_PROVIDER_NAME), BitsOfBinaryImageProvider::instance());
+    engine.addImageProvider(IMAGE_PROVIDER_NAME, new ImageProvider);
 
 #if __has_include("KCrash")
     if (QStringLiteral(BUILD_TYPE).compare(u"release", Qt::CaseInsensitive) == 0) {
@@ -484,7 +487,6 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 #endif
 
     // QML type bindings
-    qmlRegisterType<AccountQrCodeGenerator>(APPLICATION_ID, 1, 0, "AccountQrCodeGenerator");
     qmlRegisterType<StatusBar>(APPLICATION_ID, 1, 0, "StatusBar");
     qmlRegisterType<EmojiModel>(APPLICATION_ID, 1, 0, "EmojiModel");
     qmlRegisterType<EmojiProxyModel>(APPLICATION_ID, 1, 0, "EmojiProxyModel");
@@ -492,15 +494,13 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<RosterFilterModel>(APPLICATION_ID, 1, 0, "RosterFilterModel");
     qmlRegisterType<BlockingModel>(APPLICATION_ID, 1, 0, "BlockingModel");
     qmlRegisterType<BlockingWatcher>(APPLICATION_ID, 1, 0, "BlockingWatcher");
-    qmlRegisterType<ContactQrCodeGenerator>(APPLICATION_ID, 1, 0, "ContactQrCodeGenerator");
     qmlRegisterType<MessageComposition>(APPLICATION_ID, 1, 0, "MessageComposition");
     qmlRegisterType<FileSelectionModel>(APPLICATION_ID, 1, 0, "FileSelectionModel");
-    qmlRegisterType<LoginQrCodeGenerator>(APPLICATION_ID, 1, 0, "LoginQrCodeGenerator");
     qmlRegisterType<UserDevicesModel>(APPLICATION_ID, 1, 0, "UserDevicesModel");
     qmlRegisterType<CredentialsGenerator>(APPLICATION_ID, 1, 0, "CredentialsGenerator");
     qmlRegisterType<CredentialsValidator>(APPLICATION_ID, 1, 0, "CredentialsValidator");
     qmlRegisterType<RegistrationDataFormFilterModel>(APPLICATION_ID, 1, 0, "RegistrationDataFormFilterModel");
-    qmlRegisterType<ProviderListModel>(APPLICATION_ID, 1, 0, "ProviderListModel");
+    qmlRegisterType<ProviderModel>(APPLICATION_ID, 1, 0, "ProviderModel");
     qmlRegisterType<FileProgressWatcher>(APPLICATION_ID, 1, 0, "FileProgressWatcher");
     qmlRegisterType<UserResourcesWatcher>(APPLICATION_ID, 1, 0, "UserResourcesWatcher");
     qmlRegisterType<RosterItemWatcher>(APPLICATION_ID, 1, 0, "RosterItemWatcher");
@@ -515,7 +515,6 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<FileModel>(APPLICATION_ID, 1, 0, "FileModel");
     qmlRegisterType<FileProxyModel>(APPLICATION_ID, 1, 0, "FileProxyModel");
     qmlRegisterType<TextFormatter>(APPLICATION_ID, 1, 0, "TextFormatter");
-    qmlRegisterType<GroupChatQrCodeGenerator>(APPLICATION_ID, 1, 0, "GroupChatQrCodeGenerator");
     qmlRegisterType<GroupChatInviteeFilterModel>(APPLICATION_ID, 1, 0, "GroupChatInviteeFilterModel");
     qmlRegisterType<GroupChatUserModel>(APPLICATION_ID, 1, 0, "GroupChatUserModel");
     qmlRegisterType<GroupChatUserFilterModel>(APPLICATION_ID, 1, 0, "GroupChatUserFilterModel");
@@ -527,12 +526,13 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<MessageReactionModel>(APPLICATION_ID, 1, 0, "MessageReactionModel");
     qmlRegisterType<ChatController>(APPLICATION_ID, 1, 0, "ChatController");
     qmlRegisterType<VersionController>(APPLICATION_ID, 1, 0, "VersionController");
-    qmlRegisterType<AvatarCache>(APPLICATION_ID, 1, 0, "AvatarCache");
     qmlRegisterType<PresenceCache>(APPLICATION_ID, 1, 0, "PresenceCache");
+    qmlRegisterType<AvatarWatcher>(APPLICATION_ID, 1, 0, "AvatarWatcher");
+    qmlRegisterType<ProviderFilterModel>(APPLICATION_ID, 1, 0, "ProviderFilterModel");
 
     // Q_OBJECT
     qmlRegisterUncreatableType<Account>(APPLICATION_ID, 1, 0, "Account", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<AccountSettings>(APPLICATION_ID, 1, 0, "AccountSettings", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<AccountSettings>(APPLICATION_ID, 1, 0, "AccountSettings", QStringLiteral("Cannot create object"));
     qmlRegisterUncreatableType<AtmController>(APPLICATION_ID, 1, 0, "AtmController", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<QAbstractItemModel>(APPLICATION_ID, 1, 0, "QAbstractItemModel", QStringLiteral("Used by proxy models"));
     qmlRegisterUncreatableType<ClientWorker>(APPLICATION_ID, 1, 0, "ClientWorker", QStringLiteral("Cannot create object; only enums defined!"));
@@ -619,6 +619,10 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
         static auto self = new HostCompletionModel(qApp);
         return static_cast<QObject *>(self);
     });
+    qmlRegisterSingletonType<ImageProvider>(APPLICATION_ID, 1, 0, "ImageProvider", [](QQmlEngine *engine, QJSEngine *) {
+        engine->setObjectOwnership(ImageProvider::instance(), QQmlEngine::CppOwnership);
+        return ImageProvider::instance();
+    });
 
     engine.load(QUrl(QStringLiteral("qrc:/qml/main.qml")));
     if (engine.rootObjects().isEmpty())
